/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 21.06.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#ifndef _SINGLETON_H
#define _SINGLETON_H

#include "boost/thread/mutex.hpp"

// Multi-threaded singleton class template
// Create new instance only with the help of CreateInstance()
// Newer use default constructor!!! You were warned!
template < typename SingletonedClass >
class TCSingleton : private SingletonedClass
{
private:
	// Flag which indicates if m_pInstance is dead
	volatile static bool m_bIsDead;

	// That member makes TCSingleton class singleton
	// volatile is for avoiding troubles with SMP environment
	volatile static SingletonedClass* m_pInstance;

	// Mutex for synchronization access to m_pInstance
	static boost::mutex m_STMutex;

	// Section for safety (non-copyable class)
	TCSingleton();
	virtual ~TCSingleton();
	TCSingleton( const TCSingleton& );
	TCSingleton& operator=( const TCSingleton& );

	// Create instance of our singleton
	static void Create()
	{
		static SingletonedClass theInstance;
		m_pInstance = &theInstance;
	}

	// Error handling
	static void OnDeadReference()
	{
		throw std::runtime_error( "Dead reference." );
	}
	
public:
	// Acquire instance of Singleton
	static SingletonedClass& CreateInstance()
	{
		// Check if we here for the first time
		if ( !m_pInstance )
		{
			// Set lock for synchronization
			boost::mutex::scoped_lock SLock( m_STMutex );

			// Double checking for avoiding race condition
			if ( !m_pInstance )
			{
				if ( m_bIsDead )
					OnDeadReference();
				else
					Create();
			}
		}

		return ( ( SingletonedClass& )*m_pInstance );
	}
};

// Static members initialization
template < typename SingletonedClass >
volatile SingletonedClass* TCSingleton< SingletonedClass >::m_pInstance = 0;

template < typename SingletonedClass >
volatile bool TCSingleton< SingletonedClass >::m_bIsDead = false;

template < typename SingletonedClass >
boost::mutex TCSingleton< SingletonedClass >::m_STMutex;

#endif // _SINGLETON_H

