/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 13.01.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#ifndef _BOOST_ADOPT_H
#define _BOOST_ADOPT_H

#include "boost/thread/thread.hpp"
#include "boost/thread/xtime.hpp"
#include "boost/bind.hpp"

namespace boost_adopt
{

static const int NANOSEC_IN_SEC = 1000000000L;

inline void xtime_normalize( boost::xtime& time )
{
    time.sec += time.nsec / NANOSEC_IN_SEC;
    time.nsec = time.nsec % NANOSEC_IN_SEC;
}

inline void xtime_add( boost::xtime& time, int sec, int nsec = 0 )
{
    time.sec += sec;
    time.nsec += nsec;
    xtime_normalize( time );
}

inline void sleep( int sec, int nsec = 0 )
{
    boost::xtime XT;

    boost::xtime_get( &XT, boost::TIME_UTC );
    xtime_add( XT, sec, nsec );

    boost::thread::sleep( XT );
}

template< typename T, typename P >
class class_member_2_thread_adapter_
{
private:
    T* m_class_instance;
    void ( T::*m_member_fn )( P );
    P m_param;

public:
    class_member_2_thread_adapter_( T* class_instanse, void ( T::*member_fn )( P ), P param ) : 
        m_class_instance( class_instanse ),
        m_member_fn( member_fn ),
        m_param( param )
    {
    }

    void operator()()
    {                    
        // Method call
        boost::bind( m_member_fn, m_class_instance, m_param )();
    }
};

template< typename T, typename P >
class_member_2_thread_adapter_ < T, P > class_member_2_thread_adapter( T* class_instanse, 
                                                                   void ( T::*member_fn )( P ), 
                                                                   P param )
{
    return class_member_2_thread_adapter_ < T, P > ( class_instanse, member_fn, param );
}

} // namespace boost_adopt

#endif  // _BOOST_ADOPT_H
