/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 13.06.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#ifndef _PORT_PROCESSOR_H
#define _PORT_PROCESSOR_H

#include <boost/signals/connection.hpp>
#include <boost/signal.hpp>

#include "ipstream/iipstream.h"

namespace IPStream
{
	// Forward declarations
	class ipstream;

	// Interface for port processing
	// Realization must be thread-safe
	struct IPortProcessor
	{
		private:
		// Signal with no arguments and a void return value
		// Signal for stop receiving data
		boost::signal < void () > m_sglStop;

		// Indicates that processor need to stop
		volatile bool m_bNeedToStop;

		public:

		IPortProcessor() : m_bNeedToStop( false )
		{
			d_cerr( "IPortProcessor creation." );
			d_cerr( "" );
		}

		// Do we need to stop?
		inline bool ToStop()
		{
			d_cerr( "IPortProcessor::ToStop call." );
			d_cerr( "" );

			return m_bNeedToStop;
		}

		// Call processing
		virtual void operator()( ipstream& ) = 0;

		// Set the sign that processing should be stopped 
		// (for protocol state machines and other cyclic interface realizations)
		virtual inline void StopProcessing()
		{
			d_cerr( "IPortProcessor::StopProcessing call." );
			d_cerr( "" );

			m_bNeedToStop = true;

			// Send signal to all connected iipstream 
			m_sglStop();
		}

		// Method for connecting iipstream::operator() to m_sglStop
		inline void Link( IPStream::iipstream& inputStream )
		{
			d_cerr( "IPortProcessor::Link call." );
			d_cerr( "" );

			m_sglStop.connect( inputStream );
		}

		virtual ~IPortProcessor()
		{
			d_cerr( "IPortProcessor destructor." );
			d_cerr( "" );

			StopProcessing();
		};
	};

} // namespace IPStream

#endif // _PORT_PROCESSOR_H
