/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 26.05.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

// File with definitions of types for cross-platform working
// Do not include directly

#ifndef _DEFINITIONS_H
#define _DEFINITIONS_H

// Uncomment this define if you want to use prebuffered data sending
// It's more resource consuming(it uses much more threads) and slowly but more reliable(at least for TCP)
//#define _HAS_SEND_BUFFER

// Uncomment to disable the Nagle algorithm
//#define _TCP_NO_WAIT

// Uncomment this define if you want to call receive on each iipstream::operator>> call
// It is better define this macros. It will work a bit slowly but much reliable.
//#define _IGNORE_BUFSIZE

// Define this macros for controlling connection activity
//#define _KEEP_ALIVE

// Maximum block for send (0xC00) - 3Kb 
// Uncomment if you want to split send buffer before sending
//#define _SPLIT_OUT_BUF 0xC00

//use or not SSL features over TCP/IP
//#define USE_SSL
                                  
// Definition of type SOCKET
#ifdef _WIN32 // Compile under MD
	#define WIN32_LEAN_AND_MEAN     // Exclude rarely-used stuff from Windows headers
// Windows Header Files:
	#include <windows.h>
	#include <Winsock2.h>

	#ifdef IPSOCKET_EXPORTS
		#define IPSOCKET_API __declspec( dllexport )
	#else
		#define IPSOCKET_API __declspec( dllimport )
	#endif
	
// ioctl for Windows
	#define ioctl( SCKT, cmd, argp ) ioctlsocket( SCKT, cmd, argp )

	#define STARTUP WORD wVersionRequested; \
                        WSADATA wsaData; \
                        wVersionRequested = MAKEWORD( 2, 2 ); \
                        WSAStartup( wVersionRequested, &wsaData )

	#define CLEANUP WSACleanup()

	#define closeskt( sock ) ( shutdown( sock, 2 ), closesocket( sock ) )

#elif defined( __linux ) // Compile under Linux
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netdb.h>
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <sys/ioctl.h>

// Stubs for Linux
	#define IPSOCKET_API
	#define STARTUP
	#define CLEANUP 

// Linux has no default type SOCKET
typedef int SOCKET; 

// So as closesocket too
	#define closeskt( sock ) ( shutdown( sock, 2 ), close( sock ) )

	#ifndef SOMAXCONN
		// Make it like in linux by default (5 is Windows and BSD default value)
		#define SOMAXCONN 128 //5
	#endif // SOMAXCONN

#else // Unsupported platform
	#error "Unsupported platform!"
#endif // _WIN32

#ifdef _TRACABLE_E
	#include <iostream>
	#include "boost/thread/mutex.hpp"

// Uncomment this if you want to have debug output to the console instead to the file
	//#define _DEBUG_TO_CONSOLE

// Declaration of extern mutex
extern IPSOCKET_API boost::mutex gMutex;

extern IPSOCKET_API std::ostream* ostrptrOutStream;

template < typename PrintingType >
inline void Print( PrintingType ForPrinting )
{
	// Set global lock
	boost::mutex::scoped_lock BMScLock( gMutex );

	// Output bool values as "true" or "false"
	std::boolalpha( *ostrptrOutStream );

	// Output line
	if ( ostrptrOutStream -> good() )
	{
		( *ostrptrOutStream ) << ForPrinting << std::endl;
		//ostrptrOutStream -> flush();
	}
	else
		throw std::runtime_error( "Invalid output stream!" );
}

template < typename PrintingType >
inline void Print_No_NL( PrintingType ForPrinting )
{
	// Set global lock
	boost::mutex::scoped_lock BMScLock( gMutex );

	//std::boolalpha( *ostrptrOutStream );

	// Output line
	if ( ostrptrOutStream -> good() )
	{
		( *ostrptrOutStream ) << ForPrinting;
		ostrptrOutStream -> flush();
	}
	else
		throw std::runtime_error( "Invalid output stream!" );
}

	#define d_cerr_sl( ForOutput ) Print_No_NL( ForOutput )
	#define d_cerr( ForOutput ) Print( ForOutput )

#else // Compile in release mode
	#define d_cerr_sl( ForOutput )
	#define d_cerr( ForOutput ) // Nothing. What you've expected to find here guys?! :)
#endif // _TRACABLE_E        

// Section for SSL tunung
/////////////////////////////////////////////////////////////////////////////
#ifdef USE_SSL

#include "SSLcommon.h"

#define CIPHER_LIST "ALL:!ADH:!LOW:!EXP:!MD5:@STRENGTH"
#define CADIR NULL

// Global variable. Don't redefine 'em!
extern IPSOCKET_API SPTR_CSSL_CTX ptrCTX;

#define SSL_STARTUP_SERVER( tcpsrvInst, CAFILE, CERTFILESERV, CERTPASSWD, DHMINOR, DHMAJOR ) \
    assert( CSSL_CTX::init_OpenSSL() ); \
    assert( ptrCTX -> CTX_setup( \
        SSLv23_method(), \
        CERTFILESERV, \
        CAFILE, \
        CADIR, \
        CERTPASSWD, \
        SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT, \
        4, \
        SSL_OP_ALL | SSL_OP_NO_SSLv2 | SSL_OP_SINGLE_DH_USE, \
        CIPHER_LIST ) ); \
    assert( ptrCTX -> CTX_setup_DHparams( \
        DHMINOR, \
        DHMAJOR \
        ) ); \
	tcpsrvInst.initSSL( ptrCTX )

#define SSL_STARTUP_CLIENT( CAFILE, CERTFILECLI, CERTPASSWD ) \
    assert( CSSL_CTX::init_OpenSSL() ); \
	assert( ptrCTX -> CTX_setup( \
        SSLv23_method(), \
        CERTFILECLI, \
        CAFILE, \
        CADIR, \
        CERTPASSWD, \
        SSL_VERIFY_PEER, \
        4, \
        SSL_OP_ALL|SSL_OP_NO_SSLv2, \
        CIPHER_LIST ) )

#define SSL_SUTDOWN CSSL_CTX::shutdown_OpenSSL()

#else
	// Stubs for unplugged SSL
	#define SSL_STARTUP_SERVER( tcpsrvInst, CAFILE, CERTFILESERV, CERTPASSWD, DHMINOR, DHMAJOR )
	#define SSL_STARTUP_CLIENT( CAFILE, CERTFILECLI, CERTPASSWD )
    #define SSL_SUTDOWN
#endif // USE_SSL
/////////////////////////////////////////////////////////////////////////////
// End of section for SSL tunung

#endif // _DEFINITIONS_H

