/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 11.06.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#include "TCPServer.h"

#ifdef _WIN32
BOOL APIENTRY DllMain( HANDLE hModule, 
					   DWORD  ul_reason_for_call, 
					   LPVOID lpReserved
					 )
{
	switch ( ul_reason_for_call )
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}
#endif // _WIN32

namespace IPStream
{   
	namespace
	{
		const unsigned int cuiMillisecond = 1000000;

		// Number of milliseconds for Accept thread to sleep
		const unsigned int cuiAcceptSleep = 1;

		// Maximum length of accept thread queue
		const unsigned int cuiAcceptThreadLength = 512;
	}

	CTCPServer::FOR_PROCESSOR::FOR_PROCESSOR( BMutex& mtx, 
											  IPortProcessor* pprc ) :
	m_pPortProcessor( pprc ), m_bHaveToStop( false ), m_mtxInternal( mtx )
	{
		d_cerr( "CTCPServer::FOR_PROCESSOR creation." );
		d_cerr( "" );
	}

	CTCPServer::FOR_PROCESSOR::~FOR_PROCESSOR()
	{
		d_cerr( "CTCPServer::FOR_PROCESSOR destruction." );

		// If thread executes
		if ( !m_bHaveToStop && m_spThread.get() )
		{
			d_cerr( "CTCPServer::FOR_PROCESSOR destructor tryes to join thread." );

			m_bHaveToStop = true;
			m_spThread -> join();
			m_spThread.reset();

			d_cerr( "CTCPServer::FOR_PROCESSOR destructor joined thread successfully." );
		}

		// Close current ipstream
		//m_ipsTream.Close();

		d_cerr( "" );
	}

    CTCPServer::FOR_GARBAGE::FOR_GARBAGE( BMutex& mtx, FPList& prcList, BCondition& cond ) :
	m_mtxInternal( mtx ), m_lstProcessor( prcList ), m_cndQueue( cond )
	{
		d_cerr( "CTCPServer::FOR_GARBAGE creation." );
		d_cerr( "" );
	}

	CTCPServer::FOR_GARBAGE::~FOR_GARBAGE()
	{
		d_cerr( "CTCPServer::FOR_GARBAGE destruction." );
		d_cerr( "" );
	}
	
	CTCPServer::CTCPServer() : bStopGarbage( false ), m_bCopyPeerInfo( false )
	{
		m_tpProtocol = TP_TCP;

		d_cerr( "CTCPServer creation." );
		d_cerr( "" );
	}

	CTCPServer::~CTCPServer()
	{
		d_cerr( "CTCPServer destruction." );
		d_cerr( "" );
	}

	void CTCPServer::AcceptThread( ipstream strm )
	{
		d_cerr( "CTCPServer::AcceptThread." );

		// Preparations
		unsigned short uiPort;

		// Get portnumber
		strm.Get( &uiPort );

		// Temporary lock
		SrvLock TmpLock( m_mContainer );

		// Find our data
		ThreadMap::iterator tmciMyData = m_mapThreads.find( uiPort );

		// If we got no records for our portnumber - exit from this thread
		if ( tmciMyData == m_mapThreads.end() )
			throw except_ipsocket_logic_error( "CTCPServer::AcceptThread : Invalid ipstream port!" );

		// Turn reuse-addr on
		const int ciOn = 1;

		// If socket tuning operations was successfully commited
		// N.B.: IOCtl call makes soket non-blocking!
		if ( strm.SetSockOpt( ( const char* ) &ciOn, sizeof( ciOn ) ) /*&& strm.IOCtl()*/ && 
			 strm.Bind() && strm.Listen() )
		{
			d_cerr( "CTCPServer::AcceptThread Bind & Listen succeeded." );

			// Mutex for internal locking of fplTtm
			BMutex bmtxInternal;

			// Initialize temporary structure for storage in the list
			FOR_PROCESSOR fpTmp( bmtxInternal, tmciMyData -> second.m_pPortProcessor );

			// List of FOR_PROCESSOR
			FPList fplTtm;

			// Condition on queue length
			BCondition cndQueueCond;

			// Release container lock
			TmpLock.unlock();

			// Structure for creating garbage collector thread
			FOR_GARBAGE grbTmp( bmtxInternal, fplTtm, cndQueueCond );

/////////////////////////////
			// Collect garbage
			bStopGarbage = false;

			SPThread spGarbageThread(
									new boost::thread( boost_adopt::class_member_2_thread_adapter 
													   < CTCPServer, FOR_GARBAGE > 
													   ( this, 
														 &CTCPServer::GarbageCollectorThread, 
														 grbTmp ) ) );
/////////////////////////////

            // Accept thread itself
			while ( !( tmciMyData -> second.m_bHaveToStop ) )
			{
/////////////////////////////
				d_cerr( "CTCPServer::AcceptThread another one thread loop." );

				// Create new ipstream via Accept incoming connection
				ipstream ipsAccepted = strm.Accept();

				// If new ipstream is OK
				if ( ipsAccepted.good() )
				{
					d_cerr( "CTCPServer::AcceptThread Accept called." );

					// If we have to copy data about caller
					if ( m_bCopyPeerInfo )
					{
						d_cerr( "CTCPServer::AcceptThread CopyPeerInfo called." );
						strm.CopyPeerInfo( ipsAccepted );
					} // if ( m_bCopyPeerInfo )

					fpTmp.m_ipsTream = ipsAccepted;

					d_cerr( "CTCPServer::AcceptThread try to lock bmtxInternal." );

					// Set lock on fplTtm
					SrvLock SLIntLock( bmtxInternal );

					d_cerr( "CTCPServer::AcceptThread lock bmtxInternal succeeded." );

                    // Check if maximum size of queue is achieved then wait before adding new element
					while ( !( ( fplTtm.size() < cuiAcceptThreadLength ) || 
							   tmciMyData -> second.m_bHaveToStop ) )
					{   
						// For sleep for some time
                        boost::xtime xt;
						boost::xtime_get( &xt, boost::TIME_UTC );

						// Set sleep period in seconds
						xt.sec += cuiAcceptSleep;

						// Wait
						cndQueueCond.timed_wait( SLIntLock, xt );
						
					} // while

					// Add new node
					fplTtm.push_back( fpTmp );

					// Iterator to after last element
					FPList::iterator frpiLast = fplTtm.end();

					// Go to the last element
					--frpiLast;

					// Critical section
					try
					{
						// Add created thread into group
						frpiLast -> 
							m_spThread.reset(
										new boost::thread(
														 boost_adopt::class_member_2_thread_adapter 
														 < CTCPServer, FPList::iterator > 
														 ( this, 
														   &CTCPServer::ProcessorThread, 
														   frpiLast ) ) );
					}
					// Can't make new thread
					catch ( /*...*/const boost::thread_resource_error& )
					{
						d_cerr( "CTCPServer::AcceptThread boost::thread_resource_error raised while creating new processor thread." );
						//d_cerr( "CTCPServer::AcceptThread boost::exception raised while creating new processor thread." );

						// Decline call becoase can't process it
						ipsAccepted.Close();

						// Drop added node
						fplTtm.erase( frpiLast );

						// Unlock fplTtm
						SLIntLock.unlock();

						// Sleep current thread for cuiAcceptSleep*10 milliseconds
						boost_adopt::sleep( 0, cuiAcceptSleep * cuiMillisecond * 10 );

						continue;
					}

					d_cerr( "CTCPServer::AcceptThread processing thread started." );

				} // if ( ipsAccepted.good() )
				
			} // while ( !( tmciMyData -> second.m_bHaveToStop ) )

			// Stop garbage coleection
			bStopGarbage = true;

			if ( spGarbageThread.get() )
				spGarbageThread -> join();

			d_cerr( "CTCPServer::AcceptThread thread exited." );

			// If portprocessor not empty then stop all the launched threads
			if ( tmciMyData -> second.m_pPortProcessor )
				tmciMyData -> second.m_pPortProcessor -> StopProcessing();

			// Wait until all threads stop
			if ( !fplTtm.empty() )
			{
				d_cerr( "CTCPServer::AcceptThread (cleanup) thread list not empty." );

				// Visit all the nodes
				for ( FPList::iterator fpliProcessing = fplTtm.begin(); 
					fpliProcessing != fplTtm.end(); ++fpliProcessing )
				{
					d_cerr( "CTCPServer::AcceptThread (cleanup) thread have to stop." );

					// Check if thread is OK then join it
					if ( fpliProcessing -> m_spThread.get() )
						fpliProcessing -> m_spThread -> join();
				}
			}

		} // if ( strm.IOCtl() && strm.Bind() && strm.Listen() )
		else
			// This thread is joinable
			tmciMyData -> second.m_bHaveToStop = true;

		d_cerr( "" );
	}

	void CTCPServer::ProcessorThread( FPList::iterator itProc )
	{
		try
		{
			d_cerr( "CTCPServer::ProcessorThread." );

			if ( itProc -> m_pPortProcessor )
				// Call processor
				( *( itProc -> m_pPortProcessor ) )( itProc -> m_ipsTream );
		}
		// Catch possible IPSocket exceptions
		catch ( const except_ipsocket_logic_error& CaughtError )
		{
			d_cerr( typeid( CaughtError ).name() );
			d_cerr( CaughtError.what() );   
		}
		catch ( const except_e_pointer& CaughtError )
		{
			d_cerr( typeid( CaughtError ).name() );
			d_cerr( CaughtError.what() );
		}
		// Catch possible ipstreambuf exceptions
		catch ( const except_e_size& CaughtError )
		{
			d_cerr( typeid( CaughtError ).name() );
			d_cerr( CaughtError.what() );
		}
		// Catch other exceptions or they will be caught by boost::thread
		catch ( ... )
		{
			d_cerr( "Unhandled exceptions in CTCPServer::ProcessorThread!" );
		}

		// Set lock on itProc
		SrvLock SLIntLock( itProc -> m_mtxInternal );

		// End current thread
		itProc -> m_bHaveToStop = true;

		/*
		SLIntLock.unlock();
		
		// Sleep untill ipstream using may stop safely
		boost_adopt::sleep( cuiSelectTimeout );
		
		// Lock for Close call
		SLIntLock.lock();

		// Close this socket
		itProc -> m_ipsTream.Close();
		*/

		d_cerr( "CTCPServer::ProcessorThread exited." );
		d_cerr( "" );
	}

	void CTCPServer::GarbageCollectorThread( FOR_GARBAGE fgStruct )
	{
		while ( !bStopGarbage )
		{
			// Garbage collection
			// Remove nodes for finished processing threads
			if ( !fgStruct.m_lstProcessor.empty() )
			{
                // Set lock on fplTtm
				SrvLock SLIntLock( fgStruct.m_mtxInternal );

				d_cerr( "CTCPServer::GarbageCollectorThread lock bmtxInternal succeeded." );

				// Visit all the nodes
				for ( FPList::iterator fpliProcessing = fgStruct.m_lstProcessor.begin(); 
					( fpliProcessing != fgStruct.m_lstProcessor.end() ) &&
					  !bStopGarbage; )
				{
					d_cerr( "CTCPServer::GarbageCollectorThread for entered" );

					// Check if we got thread that we've to stop
					if ( fpliProcessing -> m_bHaveToStop )
					{
                        // Check if thread is OK then join it
						if ( fpliProcessing -> m_spThread.get() )
						{
                            fpliProcessing -> m_spThread -> join();
						}
                        
						d_cerr( "CTCPServer::GarbageCollectorThread processing thread joined." );

						fpliProcessing -> m_ipsTream.Close();

						// Remove this node
						fpliProcessing = fgStruct.m_lstProcessor.erase( fpliProcessing );

						d_cerr( "CTCPServer::GarbageCollectorThread processing thread stoped." );

						SLIntLock.unlock();

						// Send signal to accept thread that position is free
						fgStruct.m_cndQueue.notify_one();

						SLIntLock.lock();

						// Go to next step
						continue;

					} // if ( fpliProcessing -> m_bHaveToStop )

					++fpliProcessing;
					
				} // for

				SLIntLock.unlock();
				
			} // if ( !fgStruct.m_lstProcessor.empty() )

			// Sleep current thread for cuiAcceptSleep milliseconds
			boost_adopt::sleep( 0, cuiAcceptSleep * cuiMillisecond );

		} // while ( !bStopGarbage )

		return;
	}

	bool CTCPServer::CopyPeerInfo( bool Copy )
	{
		bool bRet = m_bCopyPeerInfo;
		m_bCopyPeerInfo = Copy;

		return bRet;
	}

} // namespace IPStream

