/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 11.06.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#ifndef _TCPSERVER_H
#define _TCPSERVER_H

#include "boost/thread/condition.hpp"

#include "Singleton/Singleton.h"

#include "interfaces/ip_server.h"

#include "interfaces/port_processor.h"

#include "ipstream/ipstream.h" 

#include <list>

#ifdef _WIN32
#ifdef TCP_EXPORTS
#define TCP_API __declspec( dllexport )
#else
#define TCP_API __declspec( dllimport )
#endif
#endif // _WIN32

namespace IPStream
{
    // Multi-threaded singleton class for processing incoming TCP connections
    class
    #ifdef _WIN32 
    TCP_API 
    #endif // _WIN32    
    CTCPServer : public ITIPServer < CTCPServer > 
    {
    private:
/**
 * Types definitions section
 */
        struct FOR_PROCESSOR
        {
            // Assotiated processor
            IPortProcessor* m_pPortProcessor;

            // Stream for passing to processor
            ipstream m_ipsTream;

            // Flag if we have to stop thread
            volatile bool m_bHaveToStop;

            // Thread
            SPThread m_spThread;

            // Reference to internal mutex
            BMutex& m_mtxInternal;

            FOR_PROCESSOR( BMutex&, IPortProcessor* pprc = 0 );

			~FOR_PROCESSOR();
        };

        typedef std::list < FOR_PROCESSOR > FPList;

		typedef boost::condition BCondition;

		struct FOR_GARBAGE
		{
			// Reference to internal mutex
            BMutex& m_mtxInternal;

			// Reference to internal list of processor threads
            FPList& m_lstProcessor;

			// Member for blocking writing to queue if it's full 
			BCondition& m_cndQueue;

			FOR_GARBAGE( BMutex&, FPList&, BCondition& );

			virtual ~FOR_GARBAGE();
		};

/**
 * End of section
 */

        // Section for safety (non-copyable class)
        CTCPServer( const CTCPServer& );
        CTCPServer& operator=( const CTCPServer& );

		// Flag for stopping garbage collection
		volatile bool bStopGarbage;

		// Flag if we need to copy peer info from accept socket to accepted one
		bool m_bCopyPeerInfo;
        
    public:

		// Accept thread
		virtual void AcceptThread( ipstream );

		// Thread for calling processor realization
		void ProcessorThread( FPList::iterator );

		// Thread for garbage collecting
		void GarbageCollectorThread( FOR_GARBAGE );

        // Never use directly
        explicit CTCPServer();
        virtual ~CTCPServer();

		// Method for managing m_bCopyPeerInfo member. Returns previous value of m_bCopyPeerInfo.
		// If Copy is false then peer info won't be copyed to newly created socket
		bool CopyPeerInfo( bool Copy = false );

    };

	typedef TCSingleton < CTCPServer > TCPServer;

} // namespace IPStream

#endif // _TCPSERVER_H

