/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 18.06.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#include "UDPServer.h"

#ifdef _WIN32
BOOL APIENTRY DllMain( HANDLE hModule, 
					  DWORD  ul_reason_for_call, 
					  LPVOID lpReserved
					  )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}
#endif // _WIN32

namespace IPStream
{
    CUDPServer::CUDPServer()
    {
		d_cerr( "CUDPServer creation." );
		d_cerr( "" );
		m_tpProtocol = TP_UDP;
    }

    CUDPServer::~CUDPServer()
    {
		d_cerr( "CUDPServer destruction." );
		d_cerr( "" );
    }

    void CUDPServer::AcceptThread( ipstream strm )
	{
		d_cerr( "CUDPServer::AcceptThread." );

		// Preparations
		unsigned short uiPort;

		// Get portnumber
		strm.Get( &uiPort );

		// Temporary lock
		SrvLock TmpLock( m_mContainer );

		// Find our data
		ThreadMap::iterator tmciMyData = m_mapThreads.find( uiPort );

		// If we got no records for our portnumber - exit from this thread
		if ( tmciMyData == m_mapThreads.end() )
			throw except_ipsocket_logic_error( "CUDPServer::AcceptThread : Invalid ipstream port!" );

		// If socket tuning operations was successfully commited
		if ( strm.Bind() )
		{
			d_cerr( "CUDPServer::AcceptThread Bind succeeded." );

			// Release container lock
			TmpLock.unlock();

			try
			{
				d_cerr( "CUDPServer::AcceptThread processor called." );

				if ( tmciMyData -> second.m_pPortProcessor )
					// Call processor
					( *( tmciMyData -> second.m_pPortProcessor ) )( strm );
			}
			// Catch possible IPSocket exceptions
			catch ( const except_ipsocket_logic_error& CaughtError )
			{
				d_cerr( typeid( CaughtError ).name() );
				d_cerr( CaughtError.what() );   
			}
			catch ( const except_e_pointer& CaughtError )
			{
				d_cerr( typeid( CaughtError ).name() );
				d_cerr( CaughtError.what() );
			}
			// Catch possible ipstreambuf exceptions
			catch ( const except_e_size& CaughtError )
			{
				d_cerr( typeid( CaughtError ).name() );
				d_cerr( CaughtError.what() );
			}
			// Catch other exceptions or they will be caught by boost::thread
			catch ( ... )
			{
				d_cerr( "Unhandled exceptions in CUDPServer::AcceptThread!" );
			}

		} // if ( strm.Bind() )

		// End current thread
		strm.Close();
		tmciMyData -> second.m_bHaveToStop = true;

		d_cerr( "CUDPServer::AcceptThread exited." );
		d_cerr( "" );
	}

} // namespace IPStream
