/////////////////////////////////////////////////////////////////////////////           
//                                                                                      
// Created by: Sergey Belovolov                                                         
// Created on: 24.06.04                                                                 
//                                                                                      
/////////////////////////////////////////////////////////////////////////////           
// (C) 2004 Sergey Belovolov                                           
// e-mail: b_sv@mail.ru
/////////////////////////////////////////////////////////////////////////////

#include "Processor_Impl2.h"

#include "ipstream/ipstream.h"

#include "SerialArray.h"

#include "boost_adopt/boost_adopt.h"

// Call processing
void Processor_Impl2::operator()( IPStream::ipstream& ipstrm )
{
	// Link stream for avoiding blocking parent thread inside this function
	Link( ipstrm );

	int iArr[ 5 ];
	unsigned int uiArrSize = sizeof( iArr );

	// Work only if array size is less then size of buffer
	if ( uiArrSize <= IPStream::cuiBufSize )
	{
		// Temporary buffer. Used by Recv
		char cInBuf[ IPStream::cuiBufSize ];

		unsigned int uiRecv = 0;

		// Receive array
		do
		{
			// Receive data & obtain number of received bites
			int iRec = ipstrm.Recv( cInBuf + uiRecv, IPStream::cuiBufSize - uiRecv );

			switch ( iRec )
			{
				// No data was received
				case IPStream::ciHasNoData:
					{
						// Sleep current thread for cuiRecvSleep milliseconds
						boost_adopt::sleep( 0, IPStream::cuiRecvSleep );

						// Next loop
						continue;
					}
					break;
					// If socket was gracefully closed
				case 0:
					// An error happend
				case -1:
					StopProcessing();
					break;
					// Received data 
				default:
					uiRecv += iRec;
			}

		} while ( !( ( uiRecv >= uiArrSize ) || ToStop() ) );

		if ( !ToStop() )
		{
			// Intermediate buffer
			IPStream::ipstreambuf ipstrbufBuffer;

			// Loop for variable length data receiving
			do
			{
				ipstrbufBuffer.Put( cInBuf, uiRecv );

				unsigned int uiBfSize = ipstrbufBuffer.Size();

				///////////
				uiRecv = 0;

				// Receive string
				do
				{
					// Receive data & obtain number of received bites
					int iRec = ipstrm.Recv( cInBuf + uiRecv, IPStream::cuiBufSize - uiRecv );

					switch ( iRec )
					{
						// No data was received
						case IPStream::ciHasNoData:
							{
								// Sleep current thread for cuiRecvSleep milliseconds
								boost_adopt::sleep( 0, IPStream::cuiRecvSleep );

								// Next loop
								continue;
							}
							break;
							// If socket was gracefully closed
						case 0:
							// An error happend
						case -1:
							StopProcessing();
							break;
							// Received data 
						default:
							uiRecv += iRec;
					}

				} while ( !( uiRecv || ( uiBfSize > uiArrSize ) || ToStop() ) );

				// Receive while we have incoming data
			} while ( ( uiRecv == IPStream::cuiBufSize ) && !ToStop() );

			if ( !ToStop() )
			{
				// Put the rest of the received data
				ipstrbufBuffer.Put( cInBuf, uiRecv );

				if ( !( ipstrbufBuffer.Empty() || ToStop() ) )
					ipstrbufBuffer >> iArr;

				if ( !( ipstrbufBuffer.Empty() || ToStop() ) )
				{
					std::string strRcv;

					ipstrbufBuffer >> strRcv;

					if ( !strRcv.empty() )
						d_cerr( strRcv );

					ipstrbufBuffer.Clear();

					ipstrbufBuffer << iArr << strRcv;

					// Loop back
					ipstrm.Send( ipstrbufBuffer.Get(), ipstrbufBuffer.Size() );
				}
			} // if ( !ToStop() )
		} // if ( !ToStop() )
	} // if ( uiArrSize <= IPStream::cuiBufSize )
}

